package dk.nsp.idsas.dao;

import dk.nsp.idsas.dao.entity.BlurringEntity;
import org.apache.log4j.Logger;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import javax.sql.DataSource;
import java.sql.Timestamp;
import java.time.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BlurringDao {
    public static int MAX_RETENTION_YEARS = 5;
    private final NamedParameterJdbcTemplate jdbcTemplate;
    private final BeanPropertyRowMapper<BlurringEntity> rowMapper;

    public BlurringDao(DataSource dataSource) {
        jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        rowMapper = BeanPropertyRowMapper.newInstance(BlurringEntity.class);
    }

    public void insert(BlurringEntity blurringEntity) {
        String query = "INSERT INTO blurring(patient_id, patient_id_type, organisation_id, organisation_id_type, expiry_date)" +
                "           VALUES (:patient_id, :patient_id_type, :organisation_id, :organisation_id_type, :expiry_date)";
        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("patient_id", blurringEntity.getPatientId());
        parameters.put("patient_id_type", blurringEntity.getPatientIdType());
        parameters.put("organisation_id", blurringEntity.getOrganisationId());
        parameters.put("organisation_id_type", blurringEntity.getOrganisationIdType());
        parameters.put("expiry_date", Timestamp.from(blurringEntity.getExpiryDate()));

        jdbcTemplate.update(query, parameters);
    }

    public List<BlurringEntity> getActiveBlurrings(String patientId, String patientIdType) {

        // Filter til kun at finde seneste blurring for en patient og en bestemt organisation
        String subQuery =
                "   SELECT id " +
                "     FROM blurring " +
                "    WHERE patient_id = :patient_id " +
                "      AND organisation_id = b.organisation_id " +
                " ORDER BY created_time DESC, id DESC " +
                "    LIMIT 1";

        // Find de seneste blurrings, der ikke er udløbet
        String query =
                " SELECT * " +
                "   FROM blurring b" +
                "  WHERE b.patient_id = :patient_id " +
                "    AND b.patient_id_type = :patient_id_type " +
                "    AND b.expiry_date > now() " +
                "    AND b.id = (" + subQuery + ") ";

        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("patient_id", patientId);
        parameters.put("patient_id_type", patientIdType);
        return jdbcTemplate.query(query, parameters, rowMapper);
    }

    public int getInactiveBlurringsCount() {
        final String query =
                "SELECT COUNT(*) AS row_count FROM blurring " +
                "WHERE expiry_date <= :retention_time ";

        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("retention_time", LocalDate.now().minusYears(MAX_RETENTION_YEARS));
        Integer count = jdbcTemplate.queryForObject(query, paramMap, Integer.class);
        return count == null ? 0 : count;
    }

    public int deleteRowsInBatch(int batchSize) {
        final String deleteQuery =
                "DELETE FROM blurring " +
                "      WHERE expiry_date <= :retention_time " +
                "      LIMIT :limit";

        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("limit", batchSize);
        paramMap.put("retention_time", LocalDate.now().minusYears(MAX_RETENTION_YEARS));
        return jdbcTemplate.update(deleteQuery, paramMap);
    }

}
