package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.CleanupJobStatus;
import org.apache.log4j.Logger;

import static dk.nsp.idsas.dao.BlurringDao.MAX_RETENTION_YEARS;

public class CleanupService {
    private static final Logger logger = Logger.getLogger(CleanupService.class);
    private static final String LOG_PREFIX = "CLEANUP JOB: ";
    private final BlurringDao blurringDao;
    private final int maxExecutionTime;
    private final int batchSize;
    private CleanupJobStatus status = CleanupJobStatus.notStarted();

    public CleanupService(BlurringDao blurringDao, int maxExecutionTime, int batchSize) {
        this.blurringDao = blurringDao;
        this.maxExecutionTime = maxExecutionTime;
        this.batchSize = batchSize;
    }

    public CleanupJobStatus getStatus() {
        return status;
    }

    public void cleanUpBlurrings() {
        cleanUpLog(String.format("Starting cleanup job. Retention time: %s years", MAX_RETENTION_YEARS));
        cleanUpLog("Inactive blurrings older than the retention time will be deleted.");

        // Hvor mange rækker skal slettes?
        final String deleteSize =
                "SELECT COUNT(*) AS row_count FROM blurring " +
                        "WHERE expiry_date <= :retention_time ";

        int inactiveBlurringsCount = blurringDao.getInactiveBlurringsCount();

        // Start slettejob
        int totalRowsDeleted = doCleanUp(maxExecutionTime, batchSize);

        // Udskriv status til loggen
        cleanUpLog(String.format("Rows deleted:         %10s", totalRowsDeleted));
        int rowsLeft = (inactiveBlurringsCount - totalRowsDeleted);
        cleanUpLog(String.format("Rows left:            %10s", rowsLeft));
        if (rowsLeft > 0) {
            cleanUpLog("Cleanup job did not finish. Please run again.");
        }

        status = new CleanupJobStatus(totalRowsDeleted, rowsLeft, rowsLeft == 0 ?
                CleanupJobStatus.ExecutionStatus.FINISHED : CleanupJobStatus.ExecutionStatus.NOT_FINISHED);
        cleanUpLog("Cleanup job finished. Status: " + status.finished);
    }

    private int doCleanUp(int maxSeconds, int batchSize) {
        long startTime = System.currentTimeMillis();
        boolean continueLoop = true;
        int totalRowsDeleted = 0;

        while (continueLoop) {
            int rowsDeleted = blurringDao.deleteRowsInBatch(batchSize);
            totalRowsDeleted += rowsDeleted;

            long currentTime = System.currentTimeMillis();
            long elapsedTime = (currentTime - startTime) / 1000; // Konverter til sekunder

            if (elapsedTime >= maxSeconds) {
                continueLoop = false;
            } else if (rowsDeleted < batchSize) {
                logger.debug("No more rows to delete.");
                continueLoop = false;
            }
        }

        return totalRowsDeleted;
    }

    private void cleanUpLog(String s) {
        logger.debug(LOG_PREFIX + s);
    }
}
